from tornado.escape import json_decode, json_encode
from profilelib.libraries.email_templates.manager import EmailTemplateManager
from profilelib.libraries.email_templates.validations import EmailTemplateValidationRules
from libraries.system.logger import Log


class ProfilelibEmailTemplate():
    def get(self, id: int=None) -> dict:
        response: dict = {}
        try:
            page = self.get_argument('page', 1)
            template_manager = EmailTemplateManager()
            response = template_manager.get_email_template(id, page)
        except Exception as e:
            Log().entry(e)
        finally:
            self.set_metadata(response['pagination'])
            self.set_data(response['data'])
            self.send()

    def post(self, param=None) -> dict:
        response: dict = {}
        try:
            data: dict = {}
            data['title'] = self.get_argument('title', None)
            data['subject'] = self.get_argument('subject', None)
            data['content'] = self.get_argument('content', None)
            data['type'] = self.get_argument('type', None)
            data['user_id'] = self.get_user('id')
            # validate form input
            validation_rules = EmailTemplateValidationRules()
            errors=validation_rules.validate_create_email_template(data)
            if len(errors):
                response['status'] = 400
                raise ValueError(errors)
            # validation end
            template_manager = EmailTemplateManager()
            response = template_manager.add_email_template(**data)
        except ValueError as e:
            self.set_response_status(self.RESPONSE_STATUS_SUCCESS_FALSE)
            self.set_status_code(400)
            self.set_message(str(e))
        except Exception as e:
            Log().entry(e)
        finally:
            self.set_data(response)
            self.send()

    def patch(self, id: int=None) -> dict:
        response: dict = {}
        try:
            data: dict = {}
            data['title'] = self.get_argument('title', None)
            data['subject'] = self.get_argument('subject', None)
            data['content'] = self.get_argument('content', None)
            data['type'] = self.get_argument('type', None)
            # validate form input
            validation_rules = EmailTemplateValidationRules()
            errors = validation_rules.validate_update_email_template(data)
            if len(errors):
                response['status'] = 400
                raise ValueError(errors)
            template_manager = EmailTemplateManager()
            response = template_manager.update_email_template(id, **data)
        except ValueError as e:
            self.set_response_status(self.RESPONSE_STATUS_SUCCESS_FALSE)
            self.set_status_code(400)
            self.set_message(str(e))
        except Exception as e:
            Log().entry(e)
        finally:
            self.set_data(response)
            self.send()

    def delete(self, id: int=None) -> str:
        response = None
        try:
            template_manager = EmailTemplateManager()
            response = template_manager.delete_email_template(id)
        except Exception as e:
            Log().entry(e)
        finally:
            self.set_response_status(response["status"])
            self.set_message(response["message"])
            self.send()