from profilelib.libraries.role.manager import RoleManager
from profilelib.libraries.role.validations.role import RoleValidationRules
from libraries.system.logger import Log

class ProfilelibRoles:
    validationRules = RoleValidationRules()
    role_manager = RoleManager()

    def get(self, id=None):
        # get all roles
        response: dict = {}
        try:
            response = self.role_manager.get_all_roles(id)
            self.set_message(response['message'])
        except:
            self.set_status_code(403)
            self.set_response_status(self.RESPONSE_STATUS_SUCCESS_FALSE)
        finally:
            if response["success_status"]:
                self.set_data(response['details'])
                self.set_response_status(self.RESPONSE_STATUS_SUCCESS_TRUE)
            else:
                self.set_response_status(self.RESPONSE_STATUS_SUCCESS_FALSE)
            self.set_status_code(response['status'])
            self.send()

    def post(self, param=None):
        response: dict ={'success_status': False}
        try:
            role: dict = {}
            role["name"] = self.get_argument('name', None)
            role["description"] = self.get_argument('description', None)
            role['type'] = self.get_argument('type', None)
            role["slug"] = role["name"].replace(' ', '_').lower()
            errors = self.validationRules.create_role_validations(role)
            if len(errors):
                response['status'] = 400
                raise ValueError(errors)
            role_manager= RoleManager()
            response = role_manager.add_role(**role)
            self.set_message(response['message'])
        except ValueError as e:
            self.set_message(str(e))
            self.set_status_code(400)

        except Exception as e:
            Log().entry(e)  # loggin error
            self.set_message(str(e))
            self.set_status_code(403)
            self.set_response_status(self.RESPONSE_STATUS_SUCCESS_FALSE)

        finally:
            if response["success_status"]:
                self.set_data(response['details'])
                self.set_response_status(self.RESPONSE_STATUS_SUCCESS_TRUE)
            else:
                self.set_response_status(self.RESPONSE_STATUS_SUCCESS_FALSE)
            self.set_status_code(response['status'])
            self.send()

    def patch(self, id=None):
        response: dict = {'success_status': False}
        try:
            role: dict = {}
            role["id"] = id
            role["name"] = self.get_argument('name', None)
            role["type"] = self.get_argument('type', None)
            role["description"] = self.get_argument('description', None)
            errors = self.validationRules.update_role_validations(role)
            if len(errors):
                response['status'] = 400
                raise ValueError(errors)
            response = self.role_manager.update_role(**role)
            self.set_message(response['message'])
        except ValueError as e:
            self.set_message(str(e))
            self.set_status_code(400)

        except Exception as e:
            Log().entry(e)  # loggin error
            self.set_message(str(e))
            self.set_status_code(403)
        finally:
            if response["success_status"]:
                self.set_data(response['details'])
                self.set_response_status(self.RESPONSE_STATUS_SUCCESS_TRUE)
            else:
                self.set_response_status(self.RESPONSE_STATUS_SUCCESS_FALSE)
            self.send()

    def delete(self, id):
        response: dict = {'success_status': False,'status':403}
        data: dict = {}
        try:
            data['id']=id
            errors = self.validationRules.check_id(data, data['id'])
            if len(errors):
                response['status'] = 400
                raise ValueError(errors)
            response = self.role_manager.delete_roles(id)
            self.set_message(response['message'])
        except ValueError as e:
            self.set_message(str(e))
            self.set_status_code(400)

        except Exception as e:
            Log().entry(e)  # loggin error
            self.set_message(str(e))
            self.set_status_code(403)
        finally:
            if response["success_status"]:
                self.set_data(response)
                self.set_response_status(self.RESPONSE_STATUS_SUCCESS_TRUE)
            else:
                self.set_response_status(self.RESPONSE_STATUS_SUCCESS_FALSE)
            self.set_data(response)
            self.send()