import tornado
import datetime
from libraries.system.manager import AbstractManager
from libraries.system.model import BaseModel
from profilelib.libraries.email_templates.model import EmailTemplateModel
from libraries.system.logger import Log


class EmailTemplateManager(AbstractManager):
    @BaseModel._meta.database.atomic()
    def get_email_template(self, id: int=None, page: int=1) -> dict:
        response :dict = {"data": [], "pagination":{}}
        try:
            if id is not None:
                template = EmailTemplateModel.get_or_none(EmailTemplateModel.id == id)
                if template is not None:
                    response["data"] = {
                        "id": template.id,
                        "title": template.title,
                        "subject": template.subject,
                        "content": template.content,
                        "created_by": template.created_by.name,
                        "type": template.type,
                        "status": template.status,
                        "created_at": template.created_at
                    }
            else:
                query = EmailTemplateModel.select().order_by(EmailTemplateModel.title.asc())
                pagination = self.get_pagination(query, page)
                templates = query.paginate(self.get_page(page), self.limit)
                data :list = []
                for template in templates:
                    data.append({
                        "id": template.id,
                        "title": template.title,
                        "subject": template.subject,
                        "content": template.content,
                        "created_by": template.created_by.name,
                        "type": template.type,
                        "status": template.status,
                        "created_at": template.created_at
                    })
                response["data"] = data
                response["pagination"] = pagination
        except Exception as e:
            Log().entry(e)
        finally:
            return response

    @BaseModel._meta.database.atomic()
    def add_email_template(self, **data: dict) -> dict:
        response :dict = {}
        try:
            email_template = EmailTemplateModel()
            email_template.title = data['title']
            email_template.subject = data['subject']
            email_template.content = data['content']
            email_template.type = data['type']
            email_template.created_by = data['user_id']
            email_template.created_at = datetime.datetime.now()
            if email_template.save():
                response = {
                    "id": email_template.id,
                    "title": email_template.title,
                    "subject": email_template.subject,
                    "content": email_template.content,
                    "type": email_template.type,
                    "created_by": email_template.created_by.name,
                    "created_at": email_template.created_at
                }
        except Exception as e:
            Log().entry(e)
        finally:
            return response

    @BaseModel._meta.database.atomic()
    def update_email_template(self, id: int, **data: dict) -> dict:
        try:
            email_template = EmailTemplateModel().get_or_none(EmailTemplateModel.id == id)
            if email_template is not None:
                email_template.title = data['title']
                email_template.subject = data['subject']
                email_template.content = data['content']
                email_template.type = data['type']
                if email_template.save():
                    response = {
                        "id": email_template.id,
                        "title": email_template.title,
                        "subject": email_template.subject,
                        "content": email_template.content,
                        "type": email_template.type,
                        "created_by": email_template.created_by.name,
                        "created_at": email_template.created_at
                    }
        except Exception as e:
            Log().entry(e)
        finally:
            return response

    @BaseModel._meta.database.atomic()
    def delete_email_template(self, id: int) -> str:
        response: str = {"status": False, "message": "Failed to delete email template."}
        try:
            delete = EmailTemplateModel.delete().where(EmailTemplateModel.id == id).execute()
            if delete:
                response["status"] = True
                response["message"] = 'Email template deleted successfully.'
        except Exception as e:
            Log().entry(e)
        finally:
            return response