import datetime
from libraries.system.manager import AbstractManager
from libraries.system.model import BaseModel
from profilelib.libraries.profile.model import UserProfileModel
from profilelib.libraries.user.model import UserModel
from libraries.system.logger import Log


class ProfileManager(AbstractManager):
    def __init__(self):
        self.response: dict = dict()
        self.response['success_status'] = False

    @BaseModel._meta.database.atomic()
    def update_profile(self, **data: dict) -> dict:
        self.response["update_profile"] = False
        self.response['message'] = "User profile could not update."
        try:
            user = UserModel.get_or_none(UserModel.id == data['user_id'])
            if user is not None:
                profiles = UserProfileModel.get_or_none(UserProfileModel.user_id == user.id)
                if profiles is not None:
                    profiles.social_media = {
                        "facebook": data["facebook"],
                        "twitter": data["twitter"],
                        "linkedin": data["linkedin"]
                        }
                    profiles.user_id = user.id
                    profiles.permanent_address = data["permanent_address"]
                    profiles.temporary_address = data["temporary_address"]
                    profiles.phone = data["phone"]
                    profiles.mobile = data["mobile"]
                    profiles.alternate_email = data["alternate_email"]
                    profiles.updated_at = datetime.datetime.now()
                    if profiles.save():
                        self.response["success_status"] = True
                        self.response["detail"] = {
                            "permanent_address": profiles.permanent_address,
                            "temporary_address": profiles.temporary_address,
                            "phone": profiles.phone,
                            "mobile": profiles.mobile,
                            "alternate_email": profiles.alternate_email,
                            "social_media": profiles.social_media
                        }
                        self.response['message'] = "User profile updated."

        except Exception as e:
            Log().entry(e)
        finally:
            return self.response

    @BaseModel._meta.database.atomic()
    def get_user_image(self, user_id):
        try:
            # get image of user
            user_profile = UserProfileModel.get_or_none(UserProfileModel.user_id == user_id)
            if user_profile is not None:
                return user_profile.image
            return
        except Exception as e:
            Log().entry(e)  # loggin error


    @BaseModel._meta.database.atomic()
    def get_preference(self, user_id=None):
        response: dict = {"view": "list", "menu_open": True}
        try:
            user_detail = UserProfileModel.get_or_none(UserProfileModel.user_id == user_id)
            if user_detail is not None:
                menu_open = user_detail.preferences['menu_open'] if 'menu_open' in user_detail.preferences else True
                view = user_detail.preferences["view"] if 'view' in user_detail.preferences else 'list'
                response = {"view": view, "menu_open": menu_open}
        except Exception as e:
            Log().entry(e)
        finally:
            return response



