from spotlight.validator import Validator
from spotlight.rules import Rule

class ProfilelibIsValidNumber(Rule):
    name = "profilelib_is_number"

    def passes(self, field: str, value: any, parameters, validator) -> bool:
        self.message_fields = dict(field=field)
        return value.isdigit()

    @property
    def message(self) -> str:
        return "{field} can only be numbers."


class ProfileValidationRules():
    def __init__(self):
        self.validator = Validator()

    def user_profile_validation(self, data: dict) -> list:
        self.validator.register_rule(ProfilelibIsValidNumber())
        rules = {
            "permanent_address": "string|max:256",
            "temporary_address": "string|max:256",
            "phone": "is_number|max:12|min:7",
            "mobile": "profilelib_is_number|max:15|min:10",
            "alternative_email": "email|max:256",
            "facebook": "url|max:256",
            "twitter": "url|max:256",
            "linkedin": "url|max:256"
        }
        self.validator.overwrite_messages = {
            "facebook.url": "Facebook url must be valid.",
            "twitter.url": "Twitter url must be valid.",
            "linkedin.url": "Facebook url must be valid.",
        }
        return self.validator.validate(data, rules)