import tornado
from libraries.system.model import BaseModel
from libraries.system.logger import Log
from libraries.system.manager import AbstractManager
from profilelib.libraries.role.model import RoleModel, UserRoleModel
from tornado.escape import json_decode, json_encode
from peewee import SQL, fn

class RoleManager(AbstractManager):
    def __init__(self):
        self.response: dict = {}
    """
        Get roles
           :parameter
            optional(id)
    """
    def get_all_roles(self, id=None):
        data: list = []
        try:
            roles = RoleModel.select().order_by(RoleModel.name)
            if id is not None:
                roles = roles.where(RoleModel.id == id)
            if len(roles):
                for role in roles:
                    data.append({
                        'id': role.id,
                        'name': role.name,
                        'description': role.description,
                        'slug': role.slug
                    })
                self.response = {
                    "details": data,
                    "success_status": True,
                    "status": 200,
                    "message": "Role list.",
                }
            else:
                self.response = {
                    "message": "Could not get roles.",
                    "success_status": False,
                    "status": 401
                }
        except Exception as e:
            Log().entry(e)  # loggin error

        finally:
            return self.response
    """
        Create roles
        :parameter
            name,description,type,slug
    """
    def add_role(self, **data):
        try:
            """
                check if slug exists
            """
            data_exists = self.check_slug(data["slug"], RoleModel)
            if data_exists is None:
                role = RoleModel(**data)
                if role.save():
                    self.response = {
                        "details":{
                            "id": role.id,
                            "name": role.name,
                            "description": role.description,
                            "slug": role.slug,
                            "created_at": role.created_at
                        },
                        "success_status": True,
                        "message": "Added role successfully.",
                        "status": 200,
                    }
            else:
                self.response = {
                    "message": "Role name already exists.",
                    "success_status": False,
                    "status": 401
                }
        except Exception as e:
            Log().entry(e)  # loggin error
        finally:
            return self.response
    """
        Update Role
        :parameter
            id,name,description,type
    """
    def update_role(self, **data):
        try:
            role = RoleModel(**data)
            if role.save():
                self.response = {
                    "details":{
                        "name": role.name,
                        "description": role.description,
                        "type" :role.type
                    },
                    "success_status": True
                }
            else:
                self.response = {
                    "message": "Could not update role.",
                    "success_status": False,
                    "status": 401

                }
        except Exception as e:
            Log().entry(e)  # loggin error
        finally:
            return self.response
    """
        Delete roles
            :parameter
               int=>id
    """
    def delete_roles(self, id):
        try:
            role = RoleModel.delete().where(RoleModel.id == id).execute()
            if role:
                self.response = {
                    "message": "Role deleted successfully.",
                    "success_status": True
                }
            else:
                self.response = {
                    "message": "Could not delete role.",
                    "success_status": False,
                    "status": 401
                }

        except Exception as e:
            Log().entry(e)  # loggin error
        finally:
            return self.response

    """
        Method for checking if the slug already exits.
    """
    def check_slug(self, slug, model):
        try:
            return model.get_or_none(model.slug == slug)
        except Exception as e:
            Log().entry(e)  # loggin error