from libraries.system.model import BaseModel
from profilelib.libraries.user.model import UserModel
from peewee import CharField, DateTimeField, datetime, IntegerField, ForeignKeyField
from playhouse.postgres_ext import JSONField, BinaryJSONField, DateTimeTZField


class RoleModel(BaseModel):
    class Meta:
        table_name = 'roles'

    role_types = {
        'default': 'System  Role',
        'app': 'Application defined role'
    }
    name = CharField(max_length=128)
    type = CharField(default='default', choices=[('default', 'System defined role'), ('app', "Application specific role")])
    slug = CharField(max_length=256, index=True)
    description = CharField(max_length=128, null=True)
    created_at = DateTimeTZField(default=datetime.datetime.now)
    updated_at = DateTimeTZField(default=datetime.datetime.now)


class UserRoleModel(BaseModel):
    class Meta:
        table_name = 'user_roles'
    user_id = ForeignKeyField(UserModel, index=True, on_delete='CASCADE')
    role_id = ForeignKeyField(RoleModel, index=True, on_delete='CASCADE')
    created_at = DateTimeTZField(default=datetime.datetime.now)
    updated_at = DateTimeTZField(default=datetime.datetime.now)