import re
from spotlight.validator import Validator
from profilelib.libraries.user.model import UserModel
from libraries.system.logger import Log
from spotlight.rules import Rule

class ProfilelibUniqueEmail(Rule):
    name = "profilelib_unique_email"

    def passes(self, field: str, value: any, parameters, validator) -> bool:
        self.message_fields = dict(field=field)
        is_unique = True

        try:
            result = UserModel.get_or_none(UserModel.email == value)
            if result is not None:
                is_unique = False
        except Exception as e:
            Log().entry(e)
        finally:
            return is_unique

    @property
    def message(self) -> str:
        return "Email already exists."


class ProfilelibComparePasswords(Rule):
    """Compare password and confirm password"""
    name = "profilelib_is_matched"

    def passes(self, field: str, value: any, parameters, validator) -> bool:
        self.message_fields = dict(field=field)
        return parameters[0].__eq__(value)

    @property
    def message(self) -> str:
        return "Password and confirm password did not matched."


class ProfilelibIsChecked(Rule):
    name = "profilelib_is_checked"

    def passes(self, field: str, value: any, parameters, validator) -> bool:
        self.message_fields = dict(field=field)
        if value == "false":
            return False
        else:
            return True

    @property
    def message(self) -> str:
        return "Please check {field}."


class UserValidationRules():
    def __init__(self):
        self.validator = Validator()

    def register_validation(self, data: dict, password: str) -> list:
        self.validator.register_rule(ProfilelibUniqueEmail())
        self.validator.register_rule(ProfilelibComparePasswords())
        self.validator.register_rule(ProfilelibIsChecked())
        rules = {
            "name": "required|string|max:250",
            "email": "required|email|profilelib_unique_email|max:250",
            "password": "required|string|min:6|max:250",
            "confirm_password": "required|string|min:6|max:250|profilelib_is_matched:"+password,
            "agree_to_terms": "required|profilelib_is_checked"
        }
        """
             if want to override error message
             add message below
             e.g  name.required:"Name is required."
        """
        self.validator.overwrite_messages = {
            "agree_to_terms": "Check I agree to the Terms of use.",
        }
        return self.validator.validate(data, rules)

    def email_validation(self, email: str):
        rules = {
            "email": "required|email|max:250",
        }
        return self.validator.validate(email, rules)

    def token_validation(self, token: str) -> list:
        rules = {
            "token": "required|string|max:250"
        }
        return self.validator.validate(token, rules)

    def update_password_validations(self, data: dict, password: str):

        rules = {
            "token": "required|string|max:250",
            "password": "required|string|min:6|max:250",
            "confirm_password": "required|string|min:6|max:250|profilelib_is_matched:" + password,
        }
        return self.validator.validate(data, rules)